% Quick and dirty processing of the raw spectra. 
% The steps are as follows:
% 1. Load the spectra (assumes XML files have been mapped to txt files)
% 2. Map the spectra onto a common m/z vector if necessary
% 3. Compute the average spectrum
% 4. Denoise the average spectrum and find peaks
% 5. Baseline correct individual spectra and extract peak heights
% 6. Save corrected spectra and peaks for later analysis

% Set logical flags for sections of the code to be run

load_spectra = 1;
map_to_common_mz = 1;
compute_avg_spectrum = 1;
denoise_and_find_peaks = 1;
  draw_denoising_fig_1 = 1;
  print_denoising_fig_1 = 1;
  draw_denoising_fig_2 = 1;
  print_denoising_fig_2 = 1;
process_individual_spectra = 1;
save_data_for_later = 1;

% 1. Load the spectra. This assumes that the XML files have been 
% and stored in RawSpectra/. See the README for more details.
if(load_spectra)
  
  txt_files = dir('RawSpectra/*.txt');
  n_spectra = length(txt_files);
  
  % The following approach dynamically allocates storage as 
  % we go, which is necessary if we don't know the sizes of 
  % the files in advance. We also acquire all of the m/z 
  % vectors in case these are initially different. 
  
  % On my Mac G4 laptop, the following script takes takes about
  % 0.6 sec for the first few dozen spectra. In cases with a 
  % few hundred spectra, the time lengthened to almost 1.2
  % sec per spectrum at the end; total run time for 156 spectra
  % is just over 2.5 minutes. 
  
  mz_matrix = 0;
  raw_intensity = 0;
  file_names = {''};
  for(i1 = 1:n_spectra)
    [mz_vec,raw_vec] = textread(...
	['RawSpectra/',txt_files(i1).name],...
	'%f%f','headerlines',1,'delimiter',',');
    mz_matrix(1:length(mz_vec),i1) = mz_vec;
    raw_intensity(1:length(raw_vec),i1) = raw_vec;
    file_names(i1) = cellstr(txt_files(i1).name);
    time_track = clock;
    [i1 time_track(4:6)]  
  end
end
  
% 2. Map the spectra onto a common m/z vector if necessary.
if(map_to_common_mz)
  
  % Many of our scripts were designed with experiments run as 
  % closely contiguous blocks in mind, and hence they implicitly
  % assume that the same m/z vector applies. If this is not the
  % case, then the data needs to be remapped so that it is. Note
  % that even here, we assume that calibration of the spectra is
  % not a major problem. This is an assumption. If data are 
  % collected at quite different times, this assumption may fail. 
  % Even if this assumption does not fail, one should be dubious
  % of peaks indentified at m/z values far from the range of the
  % calibrants used (these calibrant m/z values can be found in 
  % the xml files). 
  
  % In general, picking one m/z vector from the first few 
  % should be fine.
  %
  % As a working rule, I would suggest choosing the m/z vector
  % from the spectrum first acquired chronologically. 
  %
  % For this data set, all of the m/z vectors are the same, 
  % so this is immaterial and included solely for
  % illustration. There is yet one more point which we are 
  % glossing over here. Interpolation of the intensity values
  % onto the new m/z vector is done using simple linear 
  % interpolation, with extrapolation at the very ends if 
  % needed (be very doubtful here). Interpolation, however, 
  % assumes that we are working with function heights, and 
  % the intensity values in a SELDI spectrum are actually counts
  % corresponding to integrated function areas. This can cause
  % the weights to be slightly off if we are mapping to a 
  % qualitatively different grid (eg, equispaced on the m/z 
  % scale as opposed to equispaced on the time scale). 
  
  % Finally, there is a simpler problem -- interpolation assumes
  % that the x (m/z here) values are distinct. Since the Ciphergen
  % values are reported to 3 decimal places by default, it is 
  % possible for ties to result.
  % Checking this is fairly easy.
  % if(sum(diff(mz_matrix(:,1))==0) > 0)
  %   there is a problem
  % end
  
  % Here, we do have ties to be removed.
  
  mz_matrix(49:57,:) = repmat([11.0342; 11.0345; 11.0348; 11.0351; ...
		    11.0354; 11.0356; 11.0359; 11.0363; 11.0366], ...
			      1,n_spectra); % removing ties.
  
  mz_vec = mz_matrix(:,1);
  raw_interp = zeros(length(mz_vec),n_spectra);
  
  % about 0.15 sec/spectrum
  
  for(i1 = 1:size(raw_intensity,2))
    raw_temp = interp1(mz_matrix(:,i1),raw_intensity(:,i1),...
		       mz_vec,'linear','extrap');
    raw_interp(:,i1) = raw_temp;
    time_track = clock;
    [i1 time_track(4:6)]  
  end
end
  

% 3. Compute the average spectrum
if(compute_avg_spectrum)
  
  avg_spectrum = mean(raw_interp,2);
  
  % what, did you expect this to be hard?
end
  
% 4. Denoise the average spectrum and find peaks
if(denoise_and_find_peaks)
  
  % This step requires the use of functions from the package
  % "Cromwell" available at http://bioinformatics.mdanderson.org
  % and from the underlying scripts of the Rice Wavelet Toolbox (RWT)
  % available from http://www-dsp.rice.edu/software/rwt.shtml.
  % I assume that both have been installed in the bin/ folder.
  % The wavelet toolbox scripts use MEX scripts (C code, called
  % by Matlab) to do some of the processing. These scripts need
  % to be compiled, but there are precompiled versions available
  % at the site. I am using the precompiled scripts for Mac OS X. 
  
  addpath('bin/Cromwell');
  addpath('bin/rwt');
  
  % set the threshold level for the size of the wavelet coefficients
  % to be kept. We do not have an automatic rule for choosing this
  % value as yet, so we are currently trying several and deciding
  % which "looks best". As a heuristic, we typically see a few hundred
  % peaks in a SELDI spectrum; a thousand is too many and less than
  % 100 seems suspicious. The typical range of parameter values
  % explored is between 10 and 25; the number of values tried is 
  % usually between 3 and 5. There are a few ways of choosing this more
  % automatically that we are exploring.
  % (1) plotting the residuals (raw - smoothed) for several different 
  %     threshold values and looking for rough homoscedasticity and
  %     lack of big peaks. We have also found it useful to look at
  %     reverse residuals, defined as (smoothed - oversmoothed). 
  %     This more clearly emphasizes the emergence of peaks, as
  %     opposed to the detailed noise structure.
  % (2) using some measure of mean (median) squared error associated
  %     with the residuals
  % (3) running all threshold values between 10 and 25 in integer
  %     steps and producing a "peak tree" akin to Minnotte's 
  %     mode tree which indicates the cutoff level at which a 
  %     given peak is found
  % (4) others as we come up with them.
  % Check over at http://bioinformatics.mdanderson.org under
  % Software to see if we've come up with a better fix. 
  % We generally tune this parameter on the average spectrum, and 
  % use the value we decide on for the set. 
  
  waveletThreshold = 15; 
  %waveletThreshold = 12; 
  %waveletThreshold = 18; 
  
  % set the threshold index below which we are suspicious of the 
  % spectrum. This corresponds to regions at or below the level
  % where we think that matrix contamination may be a problem.
  % For the most part, spectra seem very jittery at the beginning,
  % as things hitting the detector interact with startup electronic
  % noise, followed by a region of near saturation as low m/z 
  % stuff overwhelms the detector, followed by a region of slowly
  % decaying baseline. We're looking for the area where this decay
  % begins. 
  %
  % As an aside, a useful guide is often supplied by the m/z values
  % and the mass range being scanned. Here, the main focus of
  % interest was the mass range up to 15kDa or so, and for this 
  % range I'd be suspicious of readings below m/z 1500 or 2000 or 
  % so. Checking the values in mz_vec, the cutoff index for m/z 
  % 1500 is 4245, and that for m/z 2000 is 4898. We plotted the 
  % values here (see below) and decided that 4500 looked reasonable. 
  
  % figure showing ranges
  if(draw_denoising_fig_1)
    
    clf;
    set(gcf,'Color','w');
    subplot(211)
    plot(avg_spectrum);
    xlabel('Time Index');
    ylabel('Average Intensity');
    title('Intensity vs Index for Pusztai Low Mass Spectra');
    
    subplot(212)
    plot(mz_vec,avg_spectrum);
    xlabel('M/Z')
    ylabel('Average Intensity');
    title('Intensity vs M/Z');
    
    if(print_denoising_fig_1) % assumes Figs/ folder exists
      
      Im = getframe(gcf);
      imwrite(Im.cdata,'Figs/avg_spectrum_to_find_cutoff_index.png');
    
    end
      
  end
  
  spectrumCutoffIndex = 4500;
  
  % The wavelet function currently expects a spectrum to be 
  % supplied as a row vector, and returns row vectors of the
  % same size. Since we produced avg_spectrum as a column 
  % vector above, we have to transpose it or the script will
  % complain (this will likely be fixed in a later version).
  
  [baseline_corrected_avg,smoothed_avg] = ...
      waveletSmoothAndBaselineCorrect(avg_spectrum',...
				      waveletThreshold,...
				      spectrumCutoffIndex);
  
  % We can then check the performance by superimposing plots
  % of the raw, smoothed, and baseline corrected versions. 
  % Typically, the differences between the raw and smoothed
  % versions will require a much closer zoom than seeing the
  % effects of the baseline subtraction. 
  
  % figure showing effects of smoothing and baseline subtraction
  if(draw_denoising_fig_2)

    clf;
    set(gcf,'Color','w');
    subplot(211)
    plot(avg_spectrum); % blue default
    hold on 
    plot(baseline_corrected_avg,'r');
    plot(smoothed_avg,'m');
    hold off
    xlabel('Time Index');
    ylabel('Intensity');
    title(['Raw (blue), Smoothed (magenta), and Baseline Corrected' ...
	   ' (red) Spectra']);
    
    subplot(212)
    plot(avg_spectrum - smoothed_avg');
    xlabel('Time Index');
    ylabel('Intensity');
    title('Raw - Smoothed Residuals');
    
    if(print_denoising_fig_2)

      Im = getframe(gcf);
      imwrite(Im.cdata,'Figs/raw_and_smoothed_avg_spectra.png');
    
    end
      
  end
  
  % get the peaks
  
  [peak_indices,left_peak_edge,right_peak_edge] = ...
      trivialPeakFinder(baseline_corrected_avg);
  n_peaks = length(peak_indices);
  
  % Trivial peak finder is _trivial_ -- it looks for local
  % maxima. Several of the peaks are likely spurious, even
  % after the denoising. These can be filtered if desired by 
  % checking the max-min intensities for a given peak interval
  % and comparing this with a local estimate of the absolute
  % residuals to impose some type of S/N cutoff. 
end
  
  
% 5. Baseline correct individual spectra and extract peak heights
if(process_individual_spectra)
  
  corrected_intensity = raw_interp; % space allocation only
  
  % about 1 sec/spectrum
  for(i1 = 1:n_spectra)
    [baseline_corrected_spectrum,smoothed_spectrum] = ...
	waveletSmoothAndBaselineCorrect(raw_interp(:,i1)',...
					waveletThreshold,...
					spectrumCutoffIndex);
    corrected_intensity(:,i1) = baseline_corrected_spectrum';
    time_track = clock;
    [i1 time_track(4:6)]  
  end
  normalization_factors = ...
      mean(corrected_intensity(spectrumCutoffIndex:end,:));
  corrected_intensity = corrected_intensity ./ ...
      repmat(normalization_factors,length(corrected_intensity),1);
  
  n_peaks = length(peak_indices);
  peak_matrix = zeros(n_peaks,n_spectra);
  for(i1 = 1:n_peaks)
    peak_matrix(i1,:) = ...
	max(corrected_intensity(...
	    left_peak_edge(i1):right_peak_edge(i1),:),[],1);
  end
  peak_mz = mz_vec(peak_indices);
  peak_mz_left = mz_vec(left_peak_edge);
  peak_mz_right = mz_vec(right_peak_edge);
  
end

% 6. Save processed spectra and peaks for later analysis
if(save_data_for_later)

  save CorrectedSpectra corrected_intensity mz_vec file_names;
  
  save Peaks peak_mz peak_mz_left peak_mz_right peak_matrix ...
      file_names;
  
  % Note, at present I am using Matlab 7, and the .mat
  % storage files it produces may not be readable in earlier
  % versions. The code should still work. 
  
  % Now we get to the real fun stuff -- grouping things into
  % their proper paired groups, and trying to identify differences
  % between (for example) pre and post chemo cancer samples. 
end
